/* CLibraryBase.h */

/* Copyright (C) 2011  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef CLIBRARYBASE_H
#define CLIBRARYBASE_H

#include "library/threads/ReloadThread.h"
#include "library/threads/ImportCachingThread.h"
#include "library/threads/ImportCopyThread.h"
#include "HelperStructs/CDirectoryReader.h"
#include "HelperStructs/MetaData.h"
#include "HelperStructs/Filter.h"
#include "DatabaseAccess/CDatabaseConnector.h"
#include "GUI/library/ImportFolderDialog/GUIImportFolder.h"
#include "application.h"

#include <QThread>
#include <QStringList>
#include <QFileSystemWatcher>




class Application;
class CLibraryBase : public QObject
{
    Q_OBJECT
public:
    CLibraryBase(Application* app, QObject *parent = 0);

    void loadDataFromDb ();


signals:
	void sig_playlist_created(QStringList&);
	void sig_track_mime_data_available(const MetaDataList&);
    void sig_all_tracks_loaded (MetaDataList&);
    void sig_all_albums_loaded(AlbumList&);
    void sig_all_artists_loaded(ArtistList&);
    void sig_tracks_for_playlist_available(MetaDataList&);
    void sig_append_tracks_to_playlist(MetaDataList&);
    void sig_mp3s_loaded(int);

    void sig_should_reload_library();
    void sig_reload_library_finished();
    void sig_reload_library_allowed(bool);
    void sig_reloading_library(QString &);
    void sig_libpath_set(QString&);

    void sig_import_result(bool);
    void sig_change_id3_tags(const MetaDataList&);

    void sig_delete_answer(QString);
    void sig_play_next_tracks(const MetaDataList&);





public slots:
    void baseDirSelected (const QString & baseDir);
    void insertMetaDataIntoDB(MetaDataList& in);

    void reloadLibrary(bool);
    void clearLibrary();
    void refresh();
    void importFiles(const QStringList&);
    void importDirectory(const QString&);

    void setLibraryPath(QString);



/* New way */
    void psl_selected_artists_changed(const QList<int>&);
    void psl_selected_albums_changed(const QList<int>&);
    void psl_disc_pressed(int);
    void psl_selected_tracks_changed(const QList<int>&);

    void psl_prepare_album_for_playlist();
    void psl_prepare_artist_for_playlist();
    void psl_prepare_track_for_playlist(int idx);

    void psl_filter_changed(const Filter&, bool force=false);
    void psl_sortorder_changed(Sort::SortOrder, Sort::SortOrder, Sort::SortOrder);
    void psl_change_id3_tags(const QList<int>& lst);

    void psl_delete_tracks(int);
    void psl_delete_certain_tracks(const QList<int>&,int);

    void psl_play_next_all_tracks();
    void psl_play_next_tracks(const QList<int>&);

    void psl_append_all_tracks();
    void psl_append_tracks(const QList<int>&);



private slots:
   void reload_thread_finished();
   void file_system_changed(const QString& path);
   void library_reloading_state_slot(QString);
   void library_reloading_state_new_block();
   
   void import_thread_finished();
   void import_thread_done();
   void import_copy_thread_finished();
   void import_dialog_opened();
   void import_dialog_closed();
   
   void accept_import(const QString&, bool);
   void cancel_import();
   void import_progress(int);


private:
    Application*        m_app;
    CDirectoryReader    m_reader;

    QString				m_library_path;
    ReloadThread* 		m_thread;
    ImportCachingThread* m_import_thread;
    ImportCopyThread*   m_import_copy_thread;
    QFileSystemWatcher*	m_watcher;
    QString		m_src_dir;
    QStringList		m_src_files;
    int			m_import_f_or_d;
    GUI_ImportFolder*   m_import_dialog;

	int					_reload_progress;

    CDatabaseConnector*	_db;

    MetaDataList        _vec_md;
    AlbumList			_vec_albums;
    ArtistList			_vec_artists;

    Sort::SortOrder		_track_sortorder;
    Sort::SortOrder		_album_sortorder;
    Sort::SortOrder		_artist_sortorder;

    QList<int>			_selected_artists;
    QList<int>			_selected_albums;
    QList<int>          _selected_tracks;

    QString             _import_to;
    bool                _import_copy;



    Filter				_filter;

    void 				emit_stuff();
    void				delete_tracks(MetaDataList& v_md, int answer);



};

#endif // CLIBRARYBASE_H
